use strict;
use warnings;
use Test::More tests => 4;

use Sub::Scheduler;
use Time::HiRes qw( sleep time );
use List::MoreUtils qw( all );


my %t;
my @check;
my $N = 5000;
my $next;

sub bench (&) {
    my $t = time;
    shift->();
    return scalar time - $t;
}

sub _add_N_tasks {
    my $t = time;
    for my $i (1 .. $N) {
        #         1  
        #       (  ).
        add_task(sub { $check[$i-1]++ },
            $t + ($i == 1 ? 1 : $i <= 11 ? 1.2 : 1.5));
    }
}

$t{add}         = bench { _add_N_tasks()                };
$t{run0}        = bench { run_tasks()                   };
$t{get_next}    = bench { $next = get_next_run_time()   };
sleep $next - time;
$t{run1}        = bench { run_tasks()                   };
is(@check, 1, '@check size=1');
sleep get_next_run_time() - time;
$t{run10}       = bench { run_tasks()                   };
is(@check, 11, '@check size=11');
sleep get_next_run_time() - time;
$t{runN}        = bench { run_tasks()                   };
is(@check, $N, '@check size=N');
$t{run0of0}     = bench { run_tasks()                   };

ok((all { $_ == 1 } @check), 'all elements in @check equal to 1');

#---  :
# add=0.0600 get_next=0.0071
# run0,1,10,N,0/0=0.0043,0.0042,0.0044,0.0894,0.000013
# 
#---  (   ()):
# add=0.220409 get_next=0.000008
# run0,1,10,N,0/0=0.000027,0.000101,0.000125,0.022650,0.000009
#
#   1 add + 1 run    30 :
#   0.0600/5000 + 0.0043    = 0.004312
#   0.2204/5000 + 0.000101  = 0.000145
#   ݣ   run    .

diag(
    sprintf "add=%.6f get_next=%.6f\nrun0,1,10,N,0/0=%.6f,%.6f,%.6f,%.6f,%.6f",
        $t{add},
        $t{get_next},
        $t{run0},
        $t{run1},
        $t{run10},
        $t{runN},
        $t{run0of0},
);

