use Test::More tests => 12;
use Test::Exception;

use Sub::Scheduler;
use Time::HiRes qw( time sleep );

sub diediedie {
    die 'diediedie?';
}

my ($time, $taskid, $next_run);

$time = time();
add_task(\&diediedie, $time);
throws_ok { run_tasks() } qr/diediedie/,    q{diediedie must die :)};
is( get_next_run_time(), undef,             q{no more tasks to run} );

$taskid = add_task(\&diediedie, $time+0.3, 0.2);
sleep get_next_run_time()-time();
throws_ok { run_tasks() } qr/diediedie/,    q{diediedie died after 0.3 sec};
$next_run =  get_next_run_time();
isnt( $next_run, undef,                     q{there are more tasks to run} );
sleep $next_run-time();
throws_ok { run_tasks() } qr/diediedie/,    q{diediedie died again after 0.2 sec};
del_task($taskid);

my %flag;
$time = time();
add_task(sub { $flag{before}++ },   $time+0.2);
add_task(\&diediedie,               $time+0.3);
add_task(sub { $flag{delayed}++ },  $time+0.4);
add_task(sub { $flag{after}++ },    $time+0.6);
throws_ok { run_tasks($time+0.5) } qr/diediedie/, q{died in the middle of run_tasks};
ok($flag{before},                           q{"before" is set});
ok(!$flag{delayed},                         q{"delayed" isn't set});
ok(!$flag{after},                           q{"after" isn't set});
lives_ok { run_tasks($time+1) }             q{run_tasks() doesn't die};
ok($flag{delayed},                          q{"delayed" is set});
ok($flag{after},                            q{"after" is set});

