use strict;
use warnings;
use Test::More tests => 20;
use Test::Exception;

use Sub::Scheduler;
use Time::HiRes qw( sleep time );

my $flag = 0;
sub flag { $flag++ }

throws_ok { add_task()                  } qr/usage: add_task/, 'add_task wrong usage';
throws_ok { add_task(\&flag)            } qr/usage: add_task/, 'add_task wrong usage';
throws_ok { add_task(\&flag, 0)         } qr/usage: add_task/, 'add_task wrong usage';
throws_ok { add_task('flag', time())    } qr/usage: add_task/, 'add_task wrong usage';
throws_ok { add_task(\&flag, time(), 0) } qr/usage: add_task/, 'add_task wrong usage';

my $taskid = add_task(\&flag, time()+0.2);
ok(defined $taskid, 'add_task returned taskid');

lives_ok { del_task($taskid); } q{del_task don't throw exception};
$taskid = add_task(\&flag, time()+0.2);

sleep 0.1;
run_tasks();
is($flag, 0, 'flag() was not executed');

sleep 0.1;
run_tasks();
is($flag, 1, 'flag() was executed');

my $next = get_next_run_time();
is($next, undef, 'no more tasks to run');

throws_ok { del_task($taskid) } qr/del_task: no such task/, q{del_task can't delete task};

my ($flag1, $flag2) = (0, 0);
sub flag1 { $flag1++; }
sub flag2 { $flag2++; }
my $time = time();
my $task1 = add_task(\&flag1, $time+0.5);
my $task2 = add_task(\&flag2, $time+0.1, 0.2);

$next = get_next_run_time();
is($next, $time+0.1, 'flag2 will be next task in 0.1 sec');

sleep 0.1;
run_tasks();
is($flag2, 1, 'flag2 was executed first time');
sleep 0.1;
run_tasks();
is($flag2, 1, 'flag2 was not executed after 0.2 sec');
sleep 0.1;
run_tasks();
is($flag2, 2, 'flag2 was executed second time');
sleep 0.1;
run_tasks();
is($flag2, 2, 'flag2 was not executed after 0.4 sec');
is($flag1, 0, 'flag1 was not executed');
sleep 0.1;
$time = time();
run_tasks($time);
is($flag1, 1, 'flag1 was executed first time');
is($flag2, 3, 'flag2 was executed third time');

$next = get_next_run_time();
is($next, $time+0.2, 'flag2 will be next task in 0.2 sec');

